#include "xiao_display.h"
#include "data.h"
#include <WiFi.h>
#include <WiFiMulti.h>
#include <WiFiClient.h>

#define SERVER_CODE

WiFiMulti wifiMulti;

const char *ssid = "FORMSHOP";
const char *password = "formshop";
const uint16_t serverPort = 80;

WiFiClient client;
#ifdef SERVER_CODE
WiFiServer server(serverPort);
#else
const char *serverAddress = "192.168.3.198"; // 服务器的IP地址
#endif

int counter = 0;
xiao_display d;

unsigned long touch_time = 0;
bool touched = false;
bool got_message = false;

unsigned long last_time = 0;
int ms, s, m, h;

// int message_types[100];
// unsigned long messag_time[100];
// int queued_message_count = 0;

void setup()
{
    Serial.begin(115200);

    if (!xiao_display_init(&d))
    {
        Serial.println("Failed to start display");
        while (1)
            ;
    }

    Serial.println("display started");
    d.gfx->fillScreen(WHITE);
    d.gfx->setCursor(10, 10);
    d.gfx->setTextColor(RED);
    d.gfx->flush();

    wifiMulti.addAP(ssid, password);
    Serial.println("Connecting to Wi-Fi...");
    while (wifiMulti.run() != WL_CONNECTED)
    {
        Serial.print(".");
        delay(500);
    }
    Serial.println("\nConnected to Wi-Fi");
    Serial.println("");
    Serial.println("WiFi connected.");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());

    // Start server
#ifdef SERVER_CODE
    server.begin();
    Serial.println("Server started");
#endif

    int code;
    if (!get_wttr2(&code, &s, &m, &h)) {
        Serial.println("Failed to get weather from internet.");
    }
}

void loop()
{
    unsigned long now = millis();
    ms += now - last_time;
    last_time = now;

    s += ms / 1000;
    ms %= 1000;
    m += s / 60;
    s %= 60;
    h += m / 60;
    h %= 24;

    uint8_t x, y;
    if (xiao_display_get_touch(&d, &x, &y))
    {
        if (now - touch_time > 1000)
        {
            touch_time = now;
            if (got_message)
            {
                got_message = false;
            }
            else
            {
                touched = true;
            }
        }
    }
    else
    {
    }

    if (!client.connected())
    {
#ifdef SERVER_CODE
        // server check new connection
        client = server.available();
#else
        Serial.println("Connecting to server...");
        if (!client.connect(serverAddress, serverPort))
        {
            Serial.println("Failed to connect");
        }
#endif
    }

    do_message();

    d.gfx->fillScreen(WHITE);
    d.gfx->draw16bitRGBBitmap(40, 40, epd_bitmap_allArray[counter], 64, 64);
    counter = (counter + 1) % 25;

    d.gfx->setCursor(120, 70);
    d.gfx->setTextColor(BLACK);
    d.gfx->setTextSize(2);
    d.gfx->printf("%d:%d:%d", h, m, s);

    if (got_message)
    {
        d.gfx->draw16bitRGBBitmap(85, 110, epd_bitmap_icons8_capybara_48, 70, 70);
    }

    // for (int i = 0; i < queued_message_count; ++i)
    // {
    //     if (millis() - last_message < 2000)
    //     {
    //         if (message_type == 1)
    //         {
    //         }
    //     }
    //     queued_message_count = 0;

        d.gfx->flush();
    }

    void do_message()
    {
        if (client.connected())
        {
            if (client.available())
            {
                String receivedMessage = client.readStringUntil('\n');

                if (receivedMessage == "touched\r")
                {
                    got_message = true;
                }

                Serial.print("MESSAGE: ");
                Serial.println(receivedMessage);
                // queued_message_count++;
            }

            if (touched)
            {
                client.println("touched");
                Serial.println("sending message");
                touched = false;
            }
        }
    }

    bool get_wttr2(int *weather_code, int *ns, int *nm, int *nh)
    {
        Serial.println("Connecting...");

        WiFiClient client;
        // WiFiClientSecure client;
        // client.setCACert(test_root_ca2);

        if (!client.connect("api.open-meteo.com", 80))
        {
            Serial.println("Failed to connect");
            return false;
        }

        client.print(
            "GET /v1/forecast?latitude=31.2222&longitude=121.4581&current=weather_code&timezone=Asia%2FShanghai&forecast_days=1 HTTP/1.1\r\n"
            "Host: api.open-meteo.com\r\n"
            "Connetion: close\r\n"
            "\r\n");

        unsigned long timeout = millis();
        while (client.available() == 0)
        {
            if (millis() - timeout > 5000)
            {
                Serial.println(">>> Client Timeout !");
                client.stop();
                return false;
            }
        }

        while (client.available())
        {
            String line = client.readStringUntil('\n');
            if (line[0] == '{')
            {
                // Serial.println(line);
                int i = line.lastIndexOf(':');
                if (i < 0)
                {
                    Serial.println("Error: no ':' in line");
                    return false;
                }
                int code = atoi(line.c_str() + i + 1);
                *weather_code = code;
            }
            else if (line.startsWith("Date: "))
            {
                const char *str = line.c_str() + 23;
                int _nh = atoi(str);
                str += 3;
                int _nm = atoi(str);
                str += 3;
                int _ns = atoi(str);
                *ns = _ns;
                *nm = _nm;
                *nh = _nh + 8;
            }
        }
        return true;
    }