#ifndef __XIAO_DISPLAY__
#define __XIAO_DISPLAY__

// https://github.com/Seeed-Studio/Seeed_Arduino_RoundDisplay/blob/main/src/lv_xiao_round_screen.h
#include <Wire.h>
#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 240
#define CHSC6X_I2C_ID 0x2e
#define CHSC6X_MAX_POINTS_NUM 1
#define CHSC6X_READ_POINT_LEN 5
#define TOUCH_INT D7
#define XIAO_DC D3
#define XIAO_CS D1

/* touch driver : chsc6x */
bool chsc6x_is_pressed(void) {
  if (digitalRead(TOUCH_INT) != LOW) {
    delay(3);
    if (digitalRead(TOUCH_INT) != LOW)
      return false;
  }
  return true;
}

void chsc6x_convert_xy(uint8_t screen_rotation, uint8_t* x, uint8_t* y) {
  if (screen_rotation) {
    uint8_t x_tmp = *x;
    uint8_t y_tmp = *y;
    uint8_t _end = (screen_rotation % 2) ? SCREEN_WIDTH : SCREEN_HEIGHT;
    *x = y_tmp;
    *y = _end - x_tmp;
  }
}

void chsc6x_get_xy(uint8_t screen_rotation, uint8_t* x, uint8_t* y) {
  uint8_t temp[CHSC6X_READ_POINT_LEN] = { 0 };
  uint8_t read_len = Wire.requestFrom(CHSC6X_I2C_ID, CHSC6X_READ_POINT_LEN);
  if (read_len == CHSC6X_READ_POINT_LEN) {
    Wire.readBytes(temp, read_len);
    if (temp[0] == 0x01) {
      chsc6x_convert_xy(screen_rotation, &temp[2], &temp[4]);
      *x = temp[2];
      *y = temp[4];
    }
  }
}

#include "I2C_BM8563.h"
#include <Arduino_GFX_Library.h>

struct xiao_display {
  Arduino_DataBus* bus;
  Arduino_GFX* gfx;
  I2C_BM8563* rtc;
  uint8_t rotation;
};

bool xiao_display_init(xiao_display* d, uint8_t screen_rotation = 0) {
  d->rotation = screen_rotation;
  d->bus = new Arduino_HWSPI(XIAO_DC, XIAO_CS);
  Arduino_GFX* output_display = new Arduino_GC9A01(d->bus, -1 /* RST */, d->rotation /* rotation */, true /* IPS */);
  d->gfx = new Arduino_Canvas(SCREEN_WIDTH /* width */, SCREEN_HEIGHT /* height */, output_display);
  d->rtc = new I2C_BM8563();
  Wire.begin();
  d->rtc->begin();
  return d->gfx->begin();
}

bool xiao_display_get_touch(xiao_display* d, uint8_t* x, uint8_t* y) {
  bool touched = chsc6x_is_pressed();
  if (touched) {
    chsc6x_get_xy(d->rotation, x, y);
  }
  return touched;
}

#endif
